@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /*
@@ -776,8 +775,9 @@ static void rfapiMonitorTimerRestart(struct rfapi_monitor_vpn *m)
 			rfapi_ntop(m->p.family, m->p.u.val, buf, BUFSIZ),
 			m->rfd->response_lifetime);
 	}
-	m->timer = thread_add_timer(bm->master, rfapiMonitorTimerExpire, m,
-				    m->rfd->response_lifetime);
+	m->timer = NULL;
+	thread_add_timer(bm->master, rfapiMonitorTimerExpire, m,
+			 m->rfd->response_lifetime, &m->timer);
 }
 
 /*
@@ -805,7 +805,7 @@ void rfapiMonitorTimersRestart(struct rfapi_descriptor *rfd, struct prefix *p)
 						 (void **)&mon_eth, &cursor)) {
 
 			if (!memcmp(mon_eth->macaddr.octet,
-				    p->u.prefix_eth.octet, ETHER_ADDR_LEN)) {
+				    p->u.prefix_eth.octet, ETH_ALEN)) {
 
 				rfapiMonitorEthTimerRestart(mon_eth);
 			}
@@ -839,7 +839,7 @@ void rfapiMonitorItNodeChanged(
 	struct bgp *bgp = bgp_get_default();
 	afi_t afi = family2afi(rn->p.family);
 #if DEBUG_L2_EXTRA
-	char buf_prefix[BUFSIZ];
+	char buf_prefix[PREFIX_STRLEN];
 #endif
 
 	assert(bgp);
@@ -848,7 +848,7 @@ void rfapiMonitorItNodeChanged(
 	nves_seen = skiplist_new(0, NULL, NULL);
 
 #if DEBUG_L2_EXTRA
-	prefix2str(&it_node->p, buf_prefix, BUFSIZ);
+	prefix2str(&it_node->p, buf_prefix, sizeof(buf_prefix));
 	vnc_zlog_debug_verbose("%s: it=%p, it_node=%p, it_node->prefix=%s",
 			       __func__, import_table, it_node, buf_prefix);
 #endif
@@ -926,22 +926,17 @@ void rfapiMonitorItNodeChanged(
 					assert(!skiplist_insert(nves_seen,
 								m->rfd, NULL));
 
-					{
-						char buf_attach_pfx[BUFSIZ];
-						char buf_target_pfx[BUFSIZ];
+					char buf_attach_pfx[PREFIX_STRLEN];
+					char buf_target_pfx[PREFIX_STRLEN];
 
-						prefix2str(&m->node->p,
-							   buf_attach_pfx,
-							   BUFSIZ);
-						prefix2str(&m->p,
-							   buf_target_pfx,
-							   BUFSIZ);
-						vnc_zlog_debug_verbose(
-							"%s: update rfd %p attached to pfx %s (targ=%s)",
-							__func__, m->rfd,
-							buf_attach_pfx,
-							buf_target_pfx);
-					}
+					prefix2str(&m->node->p, buf_attach_pfx,
+						   sizeof(buf_attach_pfx));
+					prefix2str(&m->p, buf_target_pfx,
+						   sizeof(buf_target_pfx));
+					vnc_zlog_debug_verbose(
+						"%s: update rfd %p attached to pfx %s (targ=%s)",
+						__func__, m->rfd,
+						buf_attach_pfx, buf_target_pfx);
 
 					/*
 					 * update its RIB
@@ -1099,8 +1094,9 @@ static void rfapiMonitorEthTimerRestart(struct rfapi_monitor_eth *m)
 			rfapiEthAddr2Str(&m->macaddr, buf, BUFSIZ),
 			m->rfd->response_lifetime);
 	}
-	m->timer = thread_add_timer(bm->master, rfapiMonitorEthTimerExpire, m,
-				    m->rfd->response_lifetime);
+	m->timer = NULL;
+	thread_add_timer(bm->master, rfapiMonitorEthTimerExpire, m,
+			 m->rfd->response_lifetime, &m->timer);
 }
 
 static int mon_eth_cmp(void *a, void *b)
@@ -1116,7 +1112,7 @@ static int mon_eth_cmp(void *a, void *b)
 	/*
 	 * compare ethernet addresses
 	 */
-	for (i = 0; i < ETHER_ADDR_LEN; ++i) {
+	for (i = 0; i < ETH_ALEN; ++i) {
 		if (m1->macaddr.octet[i] != m2->macaddr.octet[i])
 			return (m1->macaddr.octet[i] - m2->macaddr.octet[i]);
 	}
@@ -1270,8 +1266,9 @@ static void rfapiMonitorEthDetachImport(
 	assert(rn);
 
 #if DEBUG_L2_EXTRA
-	char buf_prefix[BUFSIZ];
-	prefix2str(&rn->p, buf_prefix, BUFSIZ);
+	char buf_prefix[PREFIX_STRLEN];
+
+	prefix2str(&rn->p, buf_prefix, sizeof(buf_prefix));
 #endif
 
 	/*
@@ -1377,6 +1374,8 @@ struct route_node *rfapiMonitorEthAdd(struct bgp *bgp,
 #if DEBUG_L2_EXTRA
 	vnc_zlog_debug_verbose("%s: inserted rfd=%p mon_eth=%p, rc=%d",
 			       __func__, rfd, val, rc);
+#else
+	(void)rc;
 #endif
 
 	/*
