@@ -33,6 +33,7 @@
 #include "lib/linklist.h"
 #include "lib/plist.h"
 #include "lib/routemap.h"
+#include "lib/lib_errors.h"
 
 #include "bgpd/bgpd.h"
 #include "bgpd/bgp_ecommunity.h"
@@ -187,7 +188,8 @@ void vnc_direct_bgp_add_route_ce(struct bgp *bgp, struct route_node *rn,
 
 
 	if (!afi) {
-		zlog_err("%s: can't get afi of route node", __func__);
+		flog_err(LIB_ERR_DEVELOPMENT,
+			  "%s: can't get afi of route node", __func__);
 		return;
 	}
 
@@ -333,7 +335,7 @@ void vnc_direct_bgp_del_route_ce(struct bgp *bgp, struct route_node *rn,
 	struct prefix ce_nexthop;
 
 	if (!afi) {
-		zlog_err("%s: bad afi", __func__);
+		flog_err(LIB_ERR_DEVELOPMENT, "%s: bad afi", __func__);
 		return;
 	}
 
@@ -402,7 +404,6 @@ void vnc_direct_bgp_del_route_ce(struct bgp *bgp, struct route_node *rn,
 
 static void vnc_direct_bgp_vpn_enable_ce(struct bgp *bgp, afi_t afi)
 {
-	struct rfapi_cfg *hc;
 	struct route_node *rn;
 	struct bgp_info *ri;
 
@@ -411,7 +412,7 @@ static void vnc_direct_bgp_vpn_enable_ce(struct bgp *bgp, afi_t afi)
 	if (!bgp)
 		return;
 
-	if (!(hc = bgp->rfapi_cfg))
+	if (!(bgp->rfapi_cfg))
 		return;
 
 	if (!VNC_EXPORT_BGP_CE_ENABLED(bgp->rfapi_cfg)) {
@@ -699,7 +700,8 @@ void vnc_direct_bgp_add_prefix(struct bgp *bgp,
 	afi_t afi = family2afi(rn->p.family);
 
 	if (!afi) {
-		zlog_err("%s: can't get afi of route node", __func__);
+		flog_err(LIB_ERR_DEVELOPMENT,
+			  "%s: can't get afi of route node", __func__);
 		return;
 	}
 
@@ -808,7 +810,8 @@ void vnc_direct_bgp_del_prefix(struct bgp *bgp,
 	afi_t afi = family2afi(rn->p.family);
 
 	if (!afi) {
-		zlog_err("%s: can't get afi route node", __func__);
+		flog_err(LIB_ERR_DEVELOPMENT, "%s: can't get afi route node",
+			  __func__);
 		return;
 	}
 
@@ -923,7 +926,8 @@ void vnc_direct_bgp_add_nve(struct bgp *bgp, struct rfapi_descriptor *rfd)
 	afi_t afi = family2afi(rfd->vn_addr.addr_family);
 
 	if (!afi) {
-		zlog_err("%s: can't get afi of nve vn addr", __func__);
+		flog_err(LIB_ERR_DEVELOPMENT,
+			  "%s: can't get afi of nve vn addr", __func__);
 		return;
 	}
 
@@ -975,7 +979,8 @@ void vnc_direct_bgp_add_nve(struct bgp *bgp, struct rfapi_descriptor *rfd)
 			if (afi == AFI_IP || afi == AFI_IP6) {
 				rt = import_table->imported_vpn[afi];
 			} else {
-				zlog_err("%s: bad afi %d", __func__, afi);
+				flog_err(LIB_ERR_DEVELOPMENT, "%s: bad afi %d",
+					  __func__, afi);
 				return;
 			}
 
@@ -1067,7 +1072,8 @@ void vnc_direct_bgp_del_nve(struct bgp *bgp, struct rfapi_descriptor *rfd)
 	afi_t afi = family2afi(rfd->vn_addr.addr_family);
 
 	if (!afi) {
-		zlog_err("%s: can't get afi of nve vn addr", __func__);
+		flog_err(LIB_ERR_DEVELOPMENT,
+			  "%s: can't get afi of nve vn addr", __func__);
 		return;
 	}
 
@@ -1114,7 +1120,8 @@ void vnc_direct_bgp_del_nve(struct bgp *bgp, struct rfapi_descriptor *rfd)
 			if (afi == AFI_IP || afi == AFI_IP6) {
 				rt = import_table->imported_vpn[afi];
 			} else {
-				zlog_err("%s: bad afi %d", __func__, afi);
+				flog_err(LIB_ERR_DEVELOPMENT, "%s: bad afi %d",
+					  __func__, afi);
 				return;
 			}
 
@@ -1289,7 +1296,7 @@ static void vnc_direct_bgp_add_group_afi(struct bgp *bgp,
 	if (afi == AFI_IP || afi == AFI_IP6) {
 		rt = import_table->imported_vpn[afi];
 	} else {
-		zlog_err("%s: bad afi %d", __func__, afi);
+		flog_err(LIB_ERR_DEVELOPMENT, "%s: bad afi %d", __func__, afi);
 		return;
 	}
 
@@ -1394,7 +1401,6 @@ static void vnc_direct_bgp_del_group_afi(struct bgp *bgp,
 		return;
 	}
 
-	assert(afi == AFI_IP || afi == AFI_IP6);
 	rt = import_table->imported_vpn[afi];
 
 	if (!rfg->nves && rfg->type != RFAPI_GROUP_CFG_VRF) {
@@ -1633,7 +1639,8 @@ void vnc_direct_bgp_rh_add_route(struct bgp *bgp, afi_t afi,
 	struct attr *iattr;
 
 	if (!afi) {
-		zlog_err("%s: can't get afi of route node", __func__);
+		flog_err(LIB_ERR_DEVELOPMENT,
+			  "%s: can't get afi of route node", __func__);
 		return;
 	}
 
@@ -1752,7 +1759,8 @@ void vnc_direct_bgp_rh_del_route(struct bgp *bgp, afi_t afi,
 	struct vnc_export_info *eti;
 
 	if (!afi) {
-		zlog_err("%s: can't get afi route node", __func__);
+		flog_err(LIB_ERR_DEVELOPMENT, "%s: can't get afi route node",
+			  __func__);
 		return;
 	}
 
