@@ -14,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -77,6 +76,7 @@ static void ospf_area_range_add(struct ospf_area *area,
 	p.family = AF_INET;
 	p.prefixlen = range->masklen;
 	p.prefix = range->addr;
+	apply_mask_ipv4(&p);
 
 	rn = route_node_get(area->ranges, (struct prefix *)&p);
 	if (rn->info)
@@ -91,7 +91,7 @@ static void ospf_area_range_delete(struct ospf_area *area,
 	struct ospf_area_range *range = rn->info;
 
 	if (range->specifics != 0)
-		ospf_delete_discard_route(area->ospf->new_table,
+		ospf_delete_discard_route(area->ospf, area->ospf->new_table,
 					  (struct prefix_ipv4 *)&rn->p);
 
 	ospf_area_range_free(range);
@@ -124,6 +124,7 @@ struct ospf_area_range *ospf_area_range_lookup_next(struct ospf_area *area,
 	p.family = AF_INET;
 	p.prefixlen = IPV4_MAX_BITLEN;
 	p.prefix = *range_net;
+	apply_mask_ipv4(&p);
 
 	if (first)
 		rn = route_top(area->ranges);
@@ -214,7 +215,7 @@ int ospf_area_range_set(struct ospf *ospf, struct in_addr area_id,
 }
 
 int ospf_area_range_cost_set(struct ospf *ospf, struct in_addr area_id,
-			     struct prefix_ipv4 *p, u_int32_t cost)
+			     struct prefix_ipv4 *p, uint32_t cost)
 {
 	struct ospf_area *area;
 	struct ospf_area_range *range;
@@ -326,8 +327,7 @@ static int ospf_abr_nssa_am_elected(struct ospf_area *area)
 	struct router_lsa *rlsa;
 	struct in_addr *best = NULL;
 
-	LSDB_LOOP(ROUTER_LSDB(area), rn, lsa)
-	{
+	LSDB_LOOP (ROUTER_LSDB(area), rn, lsa) {
 		/* sanity checks */
 		if (!lsa || (lsa->data->type != OSPF_ROUTER_LSA)
 		    || IS_LSA_SELF(lsa))
@@ -379,7 +379,7 @@ static void ospf_abr_nssa_check_status(struct ospf *ospf)
 	struct listnode *lnode, *nnode;
 
 	for (ALL_LIST_ELEMENTS(ospf->areas, lnode, nnode, area)) {
-		u_char old_state = area->NSSATranslatorState;
+		uint8_t old_state = area->NSSATranslatorState;
 
 		if (area->external_routing != OSPF_AREA_NSSA)
 			continue;
@@ -471,7 +471,7 @@ void ospf_check_abr_status(struct ospf *ospf)
 	int bb_act_attached = 0;
 	int areas_configured = 0;
 	int areas_act_attached = 0;
-	u_char new_flags = ospf->flags;
+	uint8_t new_flags = ospf->flags;
 
 	if (IS_DEBUG_OSPF_EVENT)
 		zlog_debug("ospf_check_abr_status(): Start");
@@ -585,12 +585,12 @@ static void ospf_abr_update_aggregate(struct ospf_area_range *range,
 	range->specifics++;
 }
 
-static void set_metric(struct ospf_lsa *lsa, u_int32_t metric)
+static void set_metric(struct ospf_lsa *lsa, uint32_t metric)
 {
 	struct summary_lsa *header;
-	u_char *mp;
+	uint8_t *mp;
 	metric = htonl(metric);
-	mp = (u_char *)&metric;
+	mp = (uint8_t *)&metric;
 	mp++;
 	header = (struct summary_lsa *)lsa->data;
 	memcpy(header->metric, mp, 3);
@@ -683,18 +683,18 @@ static int ospf_abr_translate_nssa(struct ospf_area *area, struct ospf_lsa *lsa)
 	return 0;
 }
 
-static void ospf_abr_translate_nssa_range(struct prefix_ipv4 *p, u_int32_t cost)
+static void ospf_abr_translate_nssa_range(struct prefix_ipv4 *p, uint32_t cost)
 {
 	/* The Type-7 is created from the aggregated prefix and forwarded
 	   for lsa installation and flooding... to be added... */
 }
 
-void ospf_abr_announce_network_to_area(struct prefix_ipv4 *p, u_int32_t cost,
+void ospf_abr_announce_network_to_area(struct prefix_ipv4 *p, uint32_t cost,
 				       struct ospf_area *area)
 {
 	struct ospf_lsa *lsa, *old = NULL;
 	struct summary_lsa *sl = NULL;
-	u_int32_t full_cost;
+	uint32_t full_cost;
 
 	if (IS_DEBUG_OSPF_EVENT)
 		zlog_debug("ospf_abr_announce_network_to_area(): Start");
@@ -976,8 +976,8 @@ static void ospf_abr_process_nssa_translates(struct ospf *ospf)
 				"looking at area %s",
 				inet_ntoa(area->area_id));
 
-		LSDB_LOOP(NSSA_LSDB(area), rn, lsa)
-		ospf_abr_translate_nssa(area, lsa);
+		LSDB_LOOP (NSSA_LSDB(area), rn, lsa)
+			ospf_abr_translate_nssa(area, lsa);
 	}
 
 	if (IS_DEBUG_OSPF_NSSA)
@@ -1089,7 +1089,7 @@ static void ospf_abr_process_network_rt(struct ospf *ospf,
 		zlog_debug("ospf_abr_process_network_rt(): Stop");
 }
 
-static void ospf_abr_announce_rtr_to_area(struct prefix_ipv4 *p, u_int32_t cost,
+static void ospf_abr_announce_rtr_to_area(struct prefix_ipv4 *p, uint32_t cost,
 					  struct ospf_area *area)
 {
 	struct ospf_lsa *lsa, *old = NULL;
@@ -1323,15 +1323,15 @@ ospf_abr_unapprove_translates(struct ospf *ospf) /* For NSSA Translations */
 	/* NSSA Translator is not checked, because it may have gone away,
 	  and we would want to flush any residuals anyway */
 
-	LSDB_LOOP(EXTERNAL_LSDB(ospf), rn, lsa)
-	if (CHECK_FLAG(lsa->flags, OSPF_LSA_LOCAL_XLT)) {
-		UNSET_FLAG(lsa->flags, OSPF_LSA_APPROVED);
-		if (IS_DEBUG_OSPF_NSSA)
-			zlog_debug(
-				"ospf_abr_unapprove_translates(): "
-				"approved unset on link id %s",
-				inet_ntoa(lsa->data->id));
-	}
+	LSDB_LOOP (EXTERNAL_LSDB(ospf), rn, lsa)
+		if (CHECK_FLAG(lsa->flags, OSPF_LSA_LOCAL_XLT)) {
+			UNSET_FLAG(lsa->flags, OSPF_LSA_APPROVED);
+			if (IS_DEBUG_OSPF_NSSA)
+				zlog_debug(
+					"ospf_abr_unapprove_translates(): "
+					"approved unset on link id %s",
+					inet_ntoa(lsa->data->id));
+		}
 
 	if (IS_DEBUG_OSPF_NSSA)
 		zlog_debug("ospf_abr_unapprove_translates(): Stop");
@@ -1353,25 +1353,25 @@ static void ospf_abr_unapprove_summaries(struct ospf *ospf)
 				"ospf_abr_unapprove_summaries(): "
 				"considering area %s",
 				inet_ntoa(area->area_id));
-		LSDB_LOOP(SUMMARY_LSDB(area), rn, lsa)
-		if (ospf_lsa_is_self_originated(ospf, lsa)) {
-			if (IS_DEBUG_OSPF_EVENT)
-				zlog_debug(
-					"ospf_abr_unapprove_summaries(): "
-					"approved unset on summary link id %s",
-					inet_ntoa(lsa->data->id));
-			UNSET_FLAG(lsa->flags, OSPF_LSA_APPROVED);
-		}
+		LSDB_LOOP (SUMMARY_LSDB(area), rn, lsa)
+			if (ospf_lsa_is_self_originated(ospf, lsa)) {
+				if (IS_DEBUG_OSPF_EVENT)
+					zlog_debug(
+						"ospf_abr_unapprove_summaries(): "
+						"approved unset on summary link id %s",
+						inet_ntoa(lsa->data->id));
+				UNSET_FLAG(lsa->flags, OSPF_LSA_APPROVED);
+			}
 
-		LSDB_LOOP(ASBR_SUMMARY_LSDB(area), rn, lsa)
-		if (ospf_lsa_is_self_originated(ospf, lsa)) {
-			if (IS_DEBUG_OSPF_EVENT)
-				zlog_debug(
-					"ospf_abr_unapprove_summaries(): "
-					"approved unset on asbr-summary link id %s",
-					inet_ntoa(lsa->data->id));
-			UNSET_FLAG(lsa->flags, OSPF_LSA_APPROVED);
-		}
+		LSDB_LOOP (ASBR_SUMMARY_LSDB(area), rn, lsa)
+			if (ospf_lsa_is_self_originated(ospf, lsa)) {
+				if (IS_DEBUG_OSPF_EVENT)
+					zlog_debug(
+						"ospf_abr_unapprove_summaries(): "
+						"approved unset on asbr-summary link id %s",
+						inet_ntoa(lsa->data->id));
+				UNSET_FLAG(lsa->flags, OSPF_LSA_APPROVED);
+			}
 	}
 
 	if (IS_DEBUG_OSPF_EVENT)
@@ -1631,8 +1631,8 @@ static void ospf_abr_remove_unapproved_translates(struct ospf *ospf)
 	if (IS_DEBUG_OSPF_NSSA)
 		zlog_debug("ospf_abr_remove_unapproved_translates(): Start");
 
-	LSDB_LOOP(EXTERNAL_LSDB(ospf), rn, lsa)
-	ospf_abr_remove_unapproved_translates_apply(ospf, lsa);
+	LSDB_LOOP (EXTERNAL_LSDB(ospf), rn, lsa)
+		ospf_abr_remove_unapproved_translates_apply(ospf, lsa);
 
 	if (IS_DEBUG_OSPF_NSSA)
 		zlog_debug("ospf_abr_remove_unapproved_translates(): Stop");
@@ -1655,15 +1655,15 @@ static void ospf_abr_remove_unapproved_summaries(struct ospf *ospf)
 				"looking at area %s",
 				inet_ntoa(area->area_id));
 
-		LSDB_LOOP(SUMMARY_LSDB(area), rn, lsa)
-		if (ospf_lsa_is_self_originated(ospf, lsa))
-			if (!CHECK_FLAG(lsa->flags, OSPF_LSA_APPROVED))
-				ospf_lsa_flush_area(lsa, area);
+		LSDB_LOOP (SUMMARY_LSDB(area), rn, lsa)
+			if (ospf_lsa_is_self_originated(ospf, lsa))
+				if (!CHECK_FLAG(lsa->flags, OSPF_LSA_APPROVED))
+					ospf_lsa_flush_area(lsa, area);
 
-		LSDB_LOOP(ASBR_SUMMARY_LSDB(area), rn, lsa)
-		if (ospf_lsa_is_self_originated(ospf, lsa))
-			if (!CHECK_FLAG(lsa->flags, OSPF_LSA_APPROVED))
-				ospf_lsa_flush_area(lsa, area);
+		LSDB_LOOP (ASBR_SUMMARY_LSDB(area), rn, lsa)
+			if (ospf_lsa_is_self_originated(ospf, lsa))
+				if (!CHECK_FLAG(lsa->flags, OSPF_LSA_APPROVED))
+					ospf_lsa_flush_area(lsa, area);
 	}
 
 	if (IS_DEBUG_OSPF_EVENT)
@@ -1684,12 +1684,13 @@ static void ospf_abr_manage_discard_routes(struct ospf *ospf)
 					       OSPF_AREA_RANGE_ADVERTISE)) {
 					if (range->specifics)
 						ospf_add_discard_route(
-							ospf->new_table, area,
+							ospf, ospf->new_table,
+							area,
 							(struct prefix_ipv4
 								 *)&rn->p);
 					else
 						ospf_delete_discard_route(
-							ospf->new_table,
+							ospf, ospf->new_table,
 							(struct prefix_ipv4
 								 *)&rn->p);
 				}
@@ -1855,7 +1856,6 @@ void ospf_schedule_abr_task(struct ospf *ospf)
 	if (IS_DEBUG_OSPF_EVENT)
 		zlog_debug("Scheduling ABR task");
 
-	if (ospf->t_abr_task == NULL)
-		ospf->t_abr_task = thread_add_timer(master, ospf_abr_task_timer,
-						    ospf, OSPF_ABR_TASK_DELAY);
+	thread_add_timer(master, ospf_abr_task_timer, ospf, OSPF_ABR_TASK_DELAY,
+			 &ospf->t_abr_task);
 }
